#version 460

// uniforms
uniform sampler2D texEarth, texSpec;
uniform float shininess = 128;

//interpolated inputs
in vec4 eye;
in vec2 tc;
in vec3 n;
in vec3 ld;

// output
out vec4 color;

void main() {

	// get texture color
	vec4 eColor = texture(texEarth, tc);

	// set the specular term to black
	vec4 spec = vec4(0.0);

	// normalize both input vectors
	vec3 nn = normalize(n);
	vec3 e = normalize(vec3(eye));

	float intensity = max(dot(nn , ld), 0.0);

	// if the vertex is lit compute the specular color
    // note: the mask is not purely binary, 
    // hence it is not possible to use it as a flag to determine if we 
    // should compute the specular component
    
	if (intensity > 0.0) {
		// compute the half vector
		vec3 h = normalize(ld + e);	
		// compute the specular intensity
		float intSpec = max(dot(h,nn), 0.0);
		// compute the specular term into spec
		spec = vec4(1) * pow(intSpec,shininess);
	}
	color = max(intensity,0.3) * eColor + spec * texture(texSpec, tc).r;
}