#version 460

const float PI = 3.14159;

// uniforms
uniform sampler2D texDay, texNight, texSpec, texClouds;
uniform float shininess = 128;
uniform float timer;

//interpolated inputs
in vec4 eye;
in vec2 tc;
in vec3 n;
in vec3 ld;

// output
out vec4 color;


float perlinNoise(vec4 pos);
float noise(in vec4 p);


void main() {

	// get texture color
	vec4 eDay = texture(texDay, tc);
	vec4 eNight = texture(texNight, tc);
   	float eSpec = texture(texSpec, tc).r;
    float eClouds = texture(texClouds, tc).r;
#if 0    
    vec2 k = tc* 16;//    + vec2(timer/10000.0, 0);
    vec2 k2 = tc*0.5;// ;
    eClouds = perlinNoise(vec4(k, timer/10000.0,0)) * perlinNoise(vec4(k2.ts, timer/50000.0,0)) ;
    eClouds = eClouds * 0.5 + 0.5;
    //eClouds =  perlinNoise(vec3(k2.ts * 0.1, k2.s * 0.71231 * 0.1 + k2.t * 0.3421* 0.1)) ;
    eClouds = smoothstep(0.5,1.0,eClouds*1.1) ;
#else
    vec2 t = tc ;
    t = vec2(2 * PI * t.s , PI * (t.t - 0.5));
    vec3 sc = vec3(sin(t.s)*cos(t.t) , sin(t.t), cos(t.s)*cos(t.t)) ;
    eClouds = perlinNoise(vec4(sc * 8, timer/10000.0)) * perlinNoise(vec4(sc*2, timer/100000.0));
    //eClouds = eClouds * 0.5 + 0.5;

    float x = 0.4;
    //eClouds = max(0.0, eClouds - (1-x)) +0.4 ;
    eClouds = smoothstep(0.1,0.7,eClouds) ;
    color = vec4(eClouds);
    //return;
#endif

	// set the specular term to black
	vec4 spec = vec4(0.0);

	// normalize both input vectors
	vec3 nn = normalize(n);
	vec3 e = normalize(vec3(eye));

	float intensity = max(dot(nn , ld), 0.0);

	// if the vertex is lit compute the specular color
    // note: the mask is not purely binary, 
    // hence it is not possible to use it as a flag to determine if we 
    // should compute the specular component
    
	if (intensity > 0.0) {
		// compute the half vector
		vec3 h = normalize(ld + e);	
		// compute the specular intensity
		float intSpec = max(dot(h,nn), 0.0);
		// compute the specular term into spec
		spec = vec4(1) * pow(intSpec,shininess);
	}

    // clouds do not reflect light
    vec4 dayColor = max(intensity,0.3) * eClouds + (1-eClouds) * (max(intensity,0.3) * eDay + spec * eSpec);
    vec4 nightColor = eNight * (1-eClouds);
    // mixing day and night colors base on intensity
    float f = smoothstep (0.0, 0.2, intensity);
    vec4 earthColor = mix(nightColor,  dayColor, f);
	color = earthColor ;
}


float snoise(vec4 p);

float perlinNoise(vec4 p) {

    float c=0, amp = 1.0;
    vec4 freq = p;


    for (int i = 0; i < 5; ++i) {
        c = c + snoise(freq) * amp;
        amp *= 0.5;
        freq *= 2;
    }

    c = c * 0.5 + 0.5;

    return c;
}
